@@ -30,18 +30,24 @@ module Options = struct
   let successes = StdOpt.store_true ()
   let failures = StdOpt.store_true ()
   let explain = StdOpt.store_true ()
-  (* let checkonly = Boilerplate.vpkglist_option () *)
+  let checkonly = Boilerplate.vpkglist_option ()
+  let latest = StdOpt.store_true ()
   let summary = StdOpt.store_true ()
   let dump = StdOpt.str_option ()
 
+  let maforeign = StdOpt.store_true ()
+
   open OptParser
   add options ~short_name:'e' ~long_name:"explain" ~help:"Explain the results" explain;
   add options ~short_name:'f' ~long_name:"failures" ~help:"Only show failures" failures;
   add options ~short_name:'s' ~long_name:"successes" ~help:"Only show successes" successes;
 
-  (* add options ~long_name:"checkonly" ~help:"Check only these package" checkonly; *)
+  add options ~long_name:"checkonly" ~help:"Check only these package" checkonly;
+  add options ~long_name:"latest" ~help:"Check only the latest version of each package" latest;
   add options ~long_name:"summary" ~help:"Print a detailed summary" summary;
 
+  add options ~long_name:"defaultedMAforeign" ~help:"Convert Arch:all packages to Multi-Arch: foreign" maforeign;
+
   add options ~long_name:"dump" ~help:"dump the cudf file" dump;
 
   include Boilerplate.MakeDistribOptions(struct let options = options end);;
@@ -57,17 +63,9 @@ let main () =
   Boilerplate.enable_debug (OptParse.Opt.get Options.verbose);
   Boilerplate.enable_timers (OptParse.Opt.get Options.timers) ["Solver"];
 
-  if not(OptParse.Opt.is_set Options.deb_native_arch) then 
-      fatal "your must specify at least the native architecture";
-
-  (*
-   let options = Options.set_options (Input.guess_format [posargs]) in
-*)
-
-  let archs = 
-    (OptParse.Opt.get Options.deb_native_arch)::
-      (OptParse.Opt.get Options.deb_foreign_arch)
-  in
+  let options = Options.set_deb_options () in
+  let target = options.Debian.Debcudf.target in
+  let builddepsarchs = Options.get_deb_buildarchs options in
 
   let pkglist, srclist =
     match posargs with
@@ -77,45 +75,67 @@ let main () =
     |l -> 
         begin match List.rev l with
         |h::t ->
-          let srclist =
-            let l = Src.input_raw [h] in
-            Src.sources2packages archs l
-          in
+          let srclist = Boilerplate.deb_load_source target builddepsarchs h in
           let pkglist = Deb.input_raw t in
           (pkglist,srclist)
         |_ -> fatal "An impossible situation occurred ?!#"
         end
   in
   let tables = Debcudf.init_tables (srclist @ pkglist) in
- 
-  let pp pkg =
-    let (p,i) = (pkg.Cudf.package,pkg.Cudf.version) in
-    let v = Debian.Debcudf.get_real_version tables (p,i) in
-    let l =
-      List.filter_map (fun k ->
-        try Some(k,Cudf.lookup_package_property pkg k)
-        with Not_found -> None
-      ) ["architecture";"source";"sourcenumber"]
-    in (p,v,l)
-  in
-  let sl = List.map (fun pkg -> Debcudf.tocudf tables pkg) srclist in
-  let l = List.fold_left (fun acc pkg -> (Debcudf.tocudf tables pkg)::acc) sl pkglist in
+  let to_cudf (p,v) = (p,Debian.Debcudf.get_cudf_version tables (p,v)) in
+  let from_cudf (p,v) = (p,Debian.Debcudf.get_real_version tables (p,v)) in
+  let pp = CudfAdd.pp from_cudf in 
 
-  let universe = Cudf.load_universe l in
+  (* XXX here latest could be a bit faster if done at the same time of the cudf
+     conversion *)
+  let sl = 
+    let l = List.map (fun pkg -> Debcudf.tocudf ~options tables pkg) srclist in
+    if OptParse.Opt.get Options.latest then
+      CudfAdd.latest l
+    else
+      l
+  in
+  let bl = 
+    List.fold_left (fun acc pkg ->
+      let pkg = 
+        if OptParse.Opt.get Options.maforeign && pkg.Packages.architecture = "all" then
+          { pkg with Packages.multiarch = `Foreign }
+        else pkg
+      in
+      (Debcudf.tocudf ~options tables pkg)::acc
+    ) sl pkglist 
+  in
+
+  let universe = Cudf.load_universe bl in
   let universe_size = Cudf.universe_size universe in
 
   let failure = OptParse.Opt.get Options.failures in
   let success = OptParse.Opt.get Options.successes in
   let explain = OptParse.Opt.get Options.explain in
   let summary = OptParse.Opt.get Options.summary in
+
+  let checklist =
+    if OptParse.Opt.is_set Options.checkonly then begin
+      List.flatten (
+        List.map (fun ((n,a),c) ->
+          let (name,filter) = Debian.Debutil.debvpkg to_cudf (("src:"^n,a),c) in
+          Cudf.lookup_packages ~filter universe name
+        ) (OptParse.Opt.get Options.checkonly)
+      )
+    end else sl
+  in
+
   let fmt = Format.std_formatter in
 
   let results = Diagnostic.default_result universe_size in
 
   Format.fprintf fmt "native-architecture: %s@." (OptParse.Opt.get Options.deb_native_arch);
 
-  if (OptParse.Opt.get Options.deb_foreign_arch) != [] then
-    Format.fprintf fmt "foreign-architecture: %s@." (String.concat "," (OptParse.Opt.get Options.deb_foreign_arch));
+  if OptParse.Opt.is_set Options.deb_foreign_archs then
+    Format.fprintf fmt "foreign-architecture: %s@." (String.concat "," (OptParse.Opt.get Options.deb_foreign_archs));
+
+  if OptParse.Opt.is_set Options.deb_target_arch then
+    Format.fprintf fmt "host-architecture: %s@." (OptParse.Opt.get Options.deb_target_arch);
 
   if failure || success then Format.fprintf fmt "@[<v 1>report:@,";
   let callback d = 
@@ -124,7 +144,7 @@ let main () =
   in
 
   Util.Timer.start timer;
-  let i = Depsolver.listcheck ~callback universe sl in
+  let i = Depsolver.listcheck ~callback universe checklist in
   ignore(Util.Timer.stop timer ());
 
   if failure || success then Format.fprintf fmt "@]@.";
@@ -141,6 +161,9 @@ let main () =
   if OptParse.Opt.is_set Options.dump then begin
     let oc = open_out (OptParse.Opt.get Options.dump) in
     info "Dumping Cudf file";
+    
+    Cudf_printer.pp_preamble oc Debcudf.preamble;
+    Printf.fprintf oc "\n";
     Cudf_printer.pp_universe oc universe
   end
 
