@@ -229,16 +229,19 @@ let outdated
       else pkg.Cudf.package
     in
     let v = 
-      if String.starts_with pkg.Cudf.package "src/" then 
-        string_of_int pkg.Cudf.version
-      else 
-        try Cudf.lookup_package_property pkg "number"
-        with Not_found ->
-          if (pkg.Cudf.version mod 2) = 1 then
-            Debian.Debcudf.get_real_version tables 
-            (pkg.Cudf.package,pkg.Cudf.version)
-          else
-            fatal "Real package without Debian Version"
+      if pkg.Cudf.version > 0 then begin
+        if String.starts_with pkg.Cudf.package "src/" then 
+          string_of_int pkg.Cudf.version
+        else 
+          try Cudf.lookup_package_property pkg "number"
+          with Not_found ->
+            if (pkg.Cudf.version mod 2) = 1 then
+              Debian.Debcudf.get_real_version tables 
+              (pkg.Cudf.package,pkg.Cudf.version)
+            else
+              fatal "Real package without Debian Version"
+      end
+      else "nan"
     in
     let l =
       List.filter_map (fun k ->
@@ -246,7 +249,7 @@ let outdated
         with Not_found -> None
       ) ["architecture";"source";"sourcenumber";"equivs"]
     in
-    (p,v,l)
+    (CudfAdd.decode p,v,l)
   in
 
   let fmt = Format.std_formatter in
@@ -281,7 +284,7 @@ let outdated
 let main () =
   let args = OptParse.OptParser.parse_argv Options.options in
   let options = 
-    match Option.get (Options.set_options Url.Deb) with
+    match Option.get (Options.set_options `Deb) with
     |Boilerplate.Deb o -> o
     |_ -> fatal "impossible"
   in
